/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 681
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "Communication.h"
#include "AD5628.h"

/******************************************************************************/
/*********************** Device configuration words ***************************/
/******************************************************************************/
#pragma config POSCMOD  = XT        /*!< XT oscillator mode selected */
#pragma config FNOSC    = PRIPLL    /*!< Primary Oscillator with PLL Module*/
#pragma config FPLLMUL  = MUL_20    /*!< 20x multiplier */
#pragma config FPLLIDIV = DIV_2     /*!< 2x divider */
#pragma config FPLLODIV = DIV_1     /*!< PLL output divided by 1 */
#pragma config FPBDIV   = DIV_1     /*!< PBCLK is SYSCLK divided by 1 */
#pragma config FWDTEN   = OFF       /*!< The WDT is not enabled */

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/
/*! List of available commands */
const char* commandsList[] = {"help?",
                              "mode=",
                              "mode?",
                              "channel=",
                              "channel?",
                              "voltage=",
                              "voltage?",
                              "register=",
                              "register?"};
const char* commandsDescription[] = {
    "  -  Displays all available commands.",
    "  -  Selects a mode of operation for the current channel. Accepted values: 0  3.",
    "  -  Displays the selected mode of operation for the current channel.",
    "  -  Selects the channel that will be affected by the other commands. Accepted values: 0  7 (one channel) or 15 (all channels).",
    "  -  Displays the selected channel.",
    "  -  Sets the output voltage for the current voltage. Accepted values: 0  2.5 (volts).",
    "  -  Displays the output voltage of the current voltage.",
    "  -  Writes the value into the Input Register of the current channel. Accepted values: 0  4095.",
    "  -  Displays the value written in the Input Register of the current channel."};
const float vrefIn = 1.25;

/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned char commandsNumber = (sizeof(commandsList) / sizeof(const char*));
unsigned char receivedCommand[20];
unsigned char invalidCommand = 0;
unsigned char commandType = 0;
unsigned char command = 0;
unsigned char displayCommand = 0;
double        commandParam = 0;

/***************************************************************************//**
 * @brief Converts a float value to a character array with 3 digits of accuracy.
 *
 * @param *buf - returns the converterd value
 * @param val - value to be converted
 *
 * @return None.
*******************************************************************************/
void FloatToString(char * buf, double val)
{
    long  intPart  = 0;
    short fracPart = 0;
    char  charPos  = 0;
    char  localBuf[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    char  i = sizeof(localBuf) - 1;

    intPart = (long)val;
    fracPart = (short)((val - intPart) * 1000 + 0.5);
    while(i > sizeof(localBuf) - 4)
    {
        localBuf[i] = (fracPart % 10) + 0x30;
        fracPart /= 10;
        i--;
    }
    localBuf[i] = '.';
    if(intPart == 0)
    {
        i --;
        localBuf[i] = '0';
    }
    while(intPart)
    {
        i --;
        localBuf[i] =(intPart % 10)  + 0x30;
        intPart /= 10;
    }
    for(charPos = i; charPos < sizeof(localBuf); charPos ++)
    {
        *buf = localBuf[charPos];
        buf ++;
    }
	*buf = 0;
}


/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
void main(void)
{
    /*! Variables holding information about the device */
    unsigned short dacRegArray[8]   = {0, 0, 0, 0, 0, 0, 0, 0};
    unsigned char  powerMode        = 0; /*!< Last powerMode value written to the device */
    unsigned char  currentChannel   = 0; /*!< Last channel selected */
    unsigned short registerValue    = 0; /*!< Last register value written to the device */
    /*! Temporary variables */
    unsigned char  tempString [10]  = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    long           tempValue  = 0;
    double         outVoltage = 0;
    double         tempFloat  = 0;
    
    /*! Initialize the UART communication peripheral. */
    UART_Init(9600);
    /*! Initialize the AD5628 device. */
    if(AD5628_Init())
    {
        CONSOLE_WriteString("AD5628 OK");
        UART_Write(0x0D);
    }
    else
    {
        CONSOLE_WriteString("AD5628 Error");
        UART_Write(0x0D);
    }
    while(1)
    {
        CONSOLE_GetCommand(receivedCommand);
        invalidCommand = 0;
        for(command = 0; command < commandsNumber; command++)
        {
            commandType = CONSOLE_CheckCommands(receivedCommand,
                                                commandsList[command],
                                                (double*)&commandParam);
            if(commandType == 0)
            {
                invalidCommand++;
            }
            if((command == 0) && (commandType != 0))    /*!< "help?" command*/
            {
                CONSOLE_WriteString("Available commands:");
                UART_Write(0x0D);
                for(displayCommand = 0; displayCommand < commandsNumber;
                    displayCommand++)
                {
                    CONSOLE_WriteString(commandsList[displayCommand]);
                    CONSOLE_WriteString(commandsDescription[displayCommand]);
                    UART_Write(0x0D);
                }
            }
            if((command == 1) && (commandType != 0))   /*!< "powerMode=" command*/
            {
                if(commandParam >=0 && commandParam <= 3)
                {
                    powerMode = (unsigned char)commandParam;
                }
                else
                {
                    if(commandParam < 0)
                    {
                        powerMode = 0;
                    }
                    else
                    {
                        powerMode = 3;
                    }
                }
                /*! Set the power mode of the selected channel. */
                AD5628_PowerMode(powerMode, currentChannel);
                /*! Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, powerMode, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 2) && (commandType != 0))   /*!< "powerMode?" command*/
            {
                /*! Send the requested value to user */
                CONSOLE_WriteString(commandsList[command - 1]);
                itoa(tempString, powerMode, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 3) && (commandType != 0))   /*!< "channel=" command*/
            {
                /*! Check if the parameter is valid */
                if((commandParam >= 0) &&    
                   (commandParam <= 7)  ||
                   (commandParam == 15))     /*!< 15 - selects all 8 channels */
                {
                    currentChannel = (char)commandParam;
                }
                else
                {
                    currentChannel = 0; /*!< Select channel A*/
                }
                /*! Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, currentChannel, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 4) && (commandType != 0))   /*!< "channel?" command*/
            {
                /*! Send the requested value to user */
                CONSOLE_WriteString(commandsList[command - 1]);
                itoa(tempString, currentChannel, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 5) && (commandType != 0))   /*!< "voltage=" command*/
            {
                outVoltage = commandParam;
                if((outVoltage >= 0) && (outVoltage <= 2.499))
                {
                    /*! Find the binary value corresponding to the output voltage*/
                    tempFloat = (outVoltage / (double)(2 * vrefIn)) * 4096;
                    registerValue = (short) tempFloat;
                    /*! Round the value */
                    if(tempFloat - registerValue >= 0.5)
                    {
                        registerValue ++;
                    }
                }
                else    /*!< Set the binary value to the upper/lower limit if
                           output voltage is out of range */
                {
                    if(outVoltage < 0)
                    {
                        registerValue = 0x0000;
                    }
                    else
                    {
                        registerValue = 0x0FFF;
                    }
                }
                dacRegArray[currentChannel] = registerValue;
                /*! Select a channel and write to its register */
                AD5628_SetInputRegister(
                                  AD5628_CMD(AD5628_CMD_WRITE_INPUT_N_UPDATE_N)|
                                  AD5628_ADDR(currentChannel)|
                                  AD5628_DATA_BITS(registerValue));
                /*! Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
                outVoltage = 2 * vrefIn * ((double)registerValue / 4096);
                /*! Convert float to string and send it to user */
                FloatToString(tempString, outVoltage);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 6) && (commandType != 0))  /*!< "voltage?" command*/
            {
                registerValue = dacRegArray[currentChannel];
                /*! Get the output voltage using data from registerValue */
                outVoltage = 2 * vrefIn * ((double)registerValue / 4096);
                /*! Send the requested value to user */
                CONSOLE_WriteString(commandsList[command - 1]);
                /*! Convert float to string and send it to user */
                FloatToString(tempString, outVoltage);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 7) && (commandType != 0))  /*!< "register=" command*/
            {
                /*! Check if parameter is valid */
                if((commandParam >= 0x0) && (commandParam <= 0xFFF))
                {
                    registerValue = (short)commandParam;
                }
                else
                {
                    if(commandParam < 0)
                    {
                        registerValue = 0x0000;
                    }
                    else
                    {
                        registerValue = 0x0FFF;
                    }
                }
                dacRegArray[currentChannel] = registerValue;
                /*! Select a channel and write to its register */
                AD5628_SetInputRegister(
                                  AD5628_CMD(AD5628_CMD_WRITE_INPUT_N_UPDATE_N)|
                                  AD5628_ADDR(currentChannel)|
                                  AD5628_DATA_BITS(registerValue));
                /*! Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, registerValue, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 8) && (commandType != 0))  /*!< "register?" command*/
            {
                registerValue = dacRegArray[currentChannel];
                /*! Send the requested value to user */
                CONSOLE_WriteString(commandsList[command - 1]);
                itoa(tempString, registerValue, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
        }
        if(invalidCommand == commandsNumber)
        {
            /*! Send feedback to user */
            CONSOLE_WriteString("Invalid command");
            UART_Write(0x0D);
        }
    }
}
